@@ -8,6 +8,7 @@ enum {
 	DCC,
 	DMM,
 	DRR,
+	DLCLS,
 	DMNCC,
 	DPAG,
 	DMSC,
@@ -21,5 +22,7 @@ enum {
 	DVLR,
 	DIUCS,
 	DBSSAP,
+	DSGS,
+	DSS,
 	Debug_LastEntry,
 };
