@@ -12,8 +12,6 @@
 #include <osmocom/gsm/tlv.h>
 #include <osmocom/gsm/protocol/gsm_04_08.h>
 
-#include <osmocom/msc/common_cs.h>
-
 struct osmo_bsc_data;
 
 struct osmo_bsc_sccp_con;
@@ -33,10 +31,4 @@ enum gsm_hooks {
 	GSM_HOOK_RR_SECURITY,
 };
 
-enum gsm_paging_event {
-	GSM_PAGING_SUCCEEDED,
-	GSM_PAGING_EXPIRED,
-	GSM_PAGING_BUSY,
-};
-
 #endif
