@@ -1,6 +1,6 @@
 /* OpenBSC SMPP 3.4 interface, SMSC-side implementation */
 
-/* (C) 2012-2013 by Harald Welte <laforge@gnumonks.org>
+/* (C) 2012-2022 by Harald Welte <laforge@gnumonks.org>
  *
  * All Rights Reserved
  *
@@ -24,6 +24,7 @@
 #include <string.h>
 #include <stdint.h>
 #include <errno.h>
+#include <time.h>
 
 #include <smpp34.h>
 #include <smpp34_structs.h>
@@ -122,6 +123,8 @@ static int smpp34_submit_tlv_msg_payload(const struct tlv_t *t,
 static int submit_to_sms(struct gsm_sms **psms, struct gsm_network *net,
 			 const struct submit_sm_t *submit)
 {
+	time_t t_now = time(NULL);
+	time_t t_validity_absolute;
 	const uint8_t *sms_msg = NULL;
 	unsigned int sms_msg_len = 0;
 	struct vlr_subscr *dest;
@@ -250,8 +253,7 @@ static int submit_to_sms(struct gsm_sms **psms, struct gsm_network *net,
 			sms_msg_len -= ud_len;
 			padbits = 7 - (ud_len % 7);
 		}
-		gsm_septets2octets(sms->user_data+ud_len, sms_msg,
-				   sms_msg_len, padbits);
+		gsm_septet_pack(sms->user_data+ud_len, sms_msg, sms_msg_len, padbits);
 		sms->user_data_len = (ud_len*8 + padbits)/7 + sms_msg_len;/* SEPTETS */
 		/* FIXME: sms->text */
 	} else {
@@ -259,6 +261,19 @@ static int submit_to_sms(struct gsm_sms **psms, struct gsm_network *net,
 		sms->user_data_len = sms_msg_len;
 	}
 
+	t_validity_absolute = smpp_parse_time_format((const char *) submit->validity_period, &t_now);
+	if (!t_validity_absolute)
+		sms->validity_minutes = net->sms_queue_cfg->default_validity_mins;
+	else
+		sms->validity_minutes = (t_validity_absolute - t_now) / 60;
+
+	if (sms->validity_minutes < net->sms_queue_cfg->minimum_validity_mins) {
+		LOGP(DLSMS, LOGL_INFO, "SMS to %s: Overriding ESME-provided validity period (%lu) "
+		     "with minimum SMSC validity period (%u) minutes\n", submit->destination_addr,
+		     sms->validity_minutes, net->sms_queue_cfg->minimum_validity_mins);
+		sms->validity_minutes = net->sms_queue_cfg->minimum_validity_mins;
+	}
+
 	*psms = sms;
 	return ESME_ROK;
 }
@@ -773,6 +788,10 @@ static int deliver_to_esme(struct osmo_esme *esme, struct gsm_sms *sms,
 	if (ret < 0)
 		return ret;
 
+	OSMO_ASSERT(!sms->smpp.esme);
+	smpp_esme_get(esme);
+	sms->smpp.esme = esme;
+
 	return smpp_cmd_enqueue(esme, vsub, sms,
 				deliver.sequence_number);
 }
