@@ -13,12 +13,14 @@
 #include <smpp34_structs.h>
 #include <smpp34_params.h>
 
-#define SMPP_SYS_ID_LEN	16
-#define SMPP_PASSWD_LEN	16
+#define SMPP_SYS_ID_LEN	15
+#define SMPP_PASSWD_LEN	8
 
 #define MODE_7BIT	7
 #define MODE_8BIT	8
 
+struct msc_a;
+
 enum esme_read_state {
 	READ_ST_IN_LEN = 0,
 	READ_ST_IN_MSG = 1,
@@ -68,6 +70,7 @@ struct osmo_smpp_acl {
 	int deliver_src_imsi;
 	int osmocom_ext;
 	int dcs_transparent;
+	int alert_notifications;
 	struct llist_head route_list;
 };
 
@@ -107,7 +110,7 @@ struct smsc {
 	struct llist_head esme_list;
 	struct llist_head acl_list;
 	struct llist_head route_list;
-	const char *bind_addr;
+	char *bind_addr;
 	uint16_t listen_port;
 	char system_id[SMPP_SYS_ID_LEN+1];
 	int accept_all;
@@ -155,13 +158,12 @@ int smpp_vty_init(void);
 
 int smpp_determine_scheme(uint8_t dcs, uint8_t *data_coding, int *mode);
 
+time_t smpp_parse_time_format(const char *vp, time_t *t_now);
 
 
 struct gsm_sms;
-struct gsm_subscriber_connection;
+struct ran_conn;
 
-int smpp_route_smpp_first(struct gsm_sms *sms,
-			    struct gsm_subscriber_connection *conn);
-int smpp_try_deliver(struct gsm_sms *sms,
-		     struct gsm_subscriber_connection *conn);
+bool smpp_route_smpp_first();
+int smpp_try_deliver(struct gsm_sms *sms, struct msc_a *msc_a);
 #endif
