@@ -246,16 +246,13 @@ static void _proc_arq_vlr_node2_post_ciph(struct osmo_fsm_inst *fi)
 {
 	struct proc_arq_priv *par = fi->priv;
 	struct vlr_subscr *vsub = par->vsub;
+	int rc;
 
 	LOGPFSM(fi, "%s()\n", __func__);
 
-	if (par->is_utran) {
-		int rc;
-		rc = par->vlr->ops.tx_common_id(par->msc_conn_ref);
-		if (rc)
-			LOGPFSML(fi, LOGL_ERROR,
-				 "Error while sending Common ID (%d)\n", rc);
-	}
+	rc = par->vlr->ops.tx_common_id(par->msc_conn_ref);
+	if (rc)
+		LOGPFSML(fi, LOGL_ERROR, "Error while sending Common ID (%d)\n", rc);
 
 	vsub->conf_by_radio_contact_ind = true;
 	if (vsub->loc_conf_in_hlr_ind == false) {
@@ -270,9 +267,12 @@ static void _proc_arq_vlr_node2_post_ciph(struct osmo_fsm_inst *fi)
 	_proc_arq_vlr_node2_post_vlr(fi);
 }
 
-static bool is_ciph_required(struct proc_arq_priv *par)
+/* Determine if sending of CMC/SMC is required */
+static bool is_cmc_smc_required(struct proc_arq_priv *par)
 {
-	return par->ciphering_required;
+	/* UTRAN: always send SecModeCmd, even if ciphering is not required.
+	 * GERAN: avoid sending CiphModeCmd if ciphering is not required. */
+	return par->is_utran || par->ciphering_required;
 }
 
 static void _proc_arq_vlr_node2(struct osmo_fsm_inst *fi)
@@ -283,7 +283,7 @@ static void _proc_arq_vlr_node2(struct osmo_fsm_inst *fi)
 
 	LOGPFSM(fi, "%s()\n", __func__);
 
-	if (!is_ciph_required(par)) {
+	if (!is_cmc_smc_required(par)) {
 		_proc_arq_vlr_node2_post_ciph(fi);
 		return;
 	}
@@ -302,7 +302,6 @@ static void _proc_arq_vlr_node2(struct osmo_fsm_inst *fi)
 	}
 
 	if (vlr_set_ciph_mode(vsub->vlr, fi, par->msc_conn_ref,
-			      par->ciphering_required,
 			      umts_aka,
 			      vsub->vlr->cfg.retrieve_imeisv_ciphered)) {
 		LOGPFSML(fi, LOGL_ERROR,
@@ -338,7 +337,7 @@ static void proc_arq_vlr_fn_post_imsi(struct osmo_fsm_inst *fi)
 	if (is_auth_required(par)) {
 		osmo_fsm_inst_state_chg(fi, PR_ARQ_S_WAIT_AUTH,
 					0, 0);
-		vsub->auth_fsm = auth_fsm_start(vsub, fi->log_level, fi,
+		vsub->auth_fsm = auth_fsm_start(vsub, fi,
 						PR_ARQ_E_AUTH_RES,
 						par->is_r99,
 						par->is_utran);
@@ -632,7 +631,7 @@ vlr_proc_acc_req(struct osmo_fsm_inst *parent,
 		 void *parent_event_data,
 		 struct vlr_instance *vlr, void *msc_conn_ref,
 		 enum vlr_parq_type type, enum osmo_cm_service_type cm_service_type,
-		 const uint8_t *mi_lv,
+		 const struct osmo_mobile_identity *mi,
 		 const struct osmo_location_area_id *lai,
 		 bool authentication_required,
 		 bool ciphering_required,
@@ -641,8 +640,6 @@ vlr_proc_acc_req(struct osmo_fsm_inst *parent,
 {
 	struct osmo_fsm_inst *fi;
 	struct proc_arq_priv *par;
-	char mi_string[GSM48_MI_SIZE];
-	uint8_t mi_type;
 
 	fi = osmo_fsm_inst_alloc_child(&proc_arq_vlr_fsm, parent,
 				       parent_event_failure);
@@ -678,16 +675,14 @@ vlr_proc_acc_req(struct osmo_fsm_inst *parent,
 		LOGPFSML(fi, LOGL_ERROR,
 			 "Authentication off on UTRAN network. Good luck.\n");
 
-	gsm48_mi_to_string(mi_string, sizeof(mi_string), mi_lv+1, mi_lv[0]);
-	mi_type = mi_lv[1] & GSM_MI_TYPE_MASK;
-	switch (mi_type) {
+	switch (mi->type) {
 	case GSM_MI_TYPE_IMSI:
-		osmo_strlcpy(par->imsi, mi_string, sizeof(par->imsi));
+		OSMO_STRLCPY_ARRAY(par->imsi, mi->imsi);
 		par->by_tmsi = false;
 		break;
 	case GSM_MI_TYPE_TMSI:
 		par->by_tmsi = true;
-		par->tmsi = osmo_load32be(mi_lv+2);
+		par->tmsi = mi->tmsi;
 		break;
 	case GSM_MI_TYPE_IMEI:
 		/* TODO: IMEI (emergency call) */
