@@ -702,10 +702,12 @@ static bool is_auth_required(struct lu_fsm_priv *lfp)
 		(lfp->ciphering_required && !auth_try_reuse_tuple(lfp->vsub, lfp->key_seq));
 }
 
-/* Determine if ciphering is required */
-static bool is_ciph_required(struct lu_fsm_priv *lfp)
+/* Determine if sending of CMC/SMC is required */
+static bool is_cmc_smc_required(struct lu_fsm_priv *lfp)
 {
-	return lfp->ciphering_required;
+	/* UTRAN: always send SecModeCmd, even if ciphering is not required.
+	 * GERAN: avoid sending CiphModeCmd if ciphering is not required. */
+	return lfp->is_utran || lfp->ciphering_required;
 }
 
 /* Determine if a HLR Update is required */
@@ -818,18 +820,15 @@ static void vlr_loc_upd_post_ciph(struct osmo_fsm_inst *fi)
 {
 	struct lu_fsm_priv *lfp = lu_fsm_fi_priv(fi);
 	struct vlr_subscr *vsub = lfp->vsub;
+	int rc;
 
 	LOGPFSM(fi, "%s()\n", __func__);
 
 	OSMO_ASSERT(vsub);
 
-	if (lfp->is_utran) {
-		int rc;
-		rc = lfp->vlr->ops.tx_common_id(lfp->msc_conn_ref);
-		if (rc)
-			LOGPFSML(fi, LOGL_ERROR,
-				 "Error while sending Common ID (%d)\n", rc);
-	}
+	rc = lfp->vlr->ops.tx_common_id(lfp->msc_conn_ref);
+	if (rc)
+		LOGPFSML(fi, LOGL_ERROR, "Error while sending Common ID (%d)\n", rc);
 
 	vsub->conf_by_radio_contact_ind = true;
 	/* Update LAI */
@@ -856,7 +855,7 @@ static void vlr_loc_upd_post_auth(struct osmo_fsm_inst *fi)
 
 	OSMO_ASSERT(vsub);
 
-	if (!is_ciph_required(lfp)) {
+	if (!is_cmc_smc_required(lfp)) {
 		vlr_loc_upd_post_ciph(fi);
 		return;
 	}
@@ -881,7 +880,6 @@ static void vlr_loc_upd_post_auth(struct osmo_fsm_inst *fi)
 	}
 
 	if (vlr_set_ciph_mode(vsub->vlr, fi, lfp->msc_conn_ref,
-			      lfp->ciphering_required,
 			      umts_aka,
 			      vsub->vlr->cfg.retrieve_imeisv_ciphered)) {
 		LOGPFSML(fi, LOGL_ERROR,
@@ -906,7 +904,7 @@ static void vlr_loc_upd_node1(struct osmo_fsm_inst *fi)
 		/* Authenticate_VLR */
 		osmo_fsm_inst_state_chg(fi, VLR_ULA_S_WAIT_AUTH,
 					LU_TIMEOUT_LONG, 0);
-		vsub->auth_fsm = auth_fsm_start(lfp->vsub, fi->log_level,
+		vsub->auth_fsm = auth_fsm_start(lfp->vsub,
 						fi, VLR_ULA_E_AUTH_RES,
 						lfp->is_r99,
 						lfp->is_utran);
