@@ -45,6 +45,7 @@
 #include "msc_vlr_tests.h"
 
 void *msc_vlr_tests_ctx = NULL;
+void *msgb_ctx = NULL;
 
 bool _log_lines = false;
 
@@ -985,12 +986,13 @@ void ms_sends_ciphering_mode_complete(const char *inner_ran_msg)
 		g_msub = NULL;
 }
 
-void ms_sends_security_mode_complete()
+void ms_sends_security_mode_complete(uint8_t utran_encryption)
 {
 	struct ran_msg ran_dec;
 
 	ran_dec = (struct ran_msg){
 		.msg_type = RAN_MSG_CIPHER_MODE_COMPLETE,
+		.cipher_mode_complete.utran_encryption = utran_encryption,
 	};
 	fake_msc_a_ran_dec(&ran_dec);
 
@@ -1005,6 +1007,7 @@ void ms_sends_assignment_complete(enum mgcp_codecs assigned_codec)
 	ran_dec = (struct ran_msg){
 		.msg_type = RAN_MSG_ASSIGNMENT_COMPLETE,
 		.assignment_complete = {
+			.codec_present = true,
 			.codec = assigned_codec,
 		},
 	};
@@ -1043,40 +1046,6 @@ void fake_time_start()
 	fake_time_passes(0, 0);
 }
 
-static void check_talloc(void *msgb_ctx, void *msc_vlr_tests_ctx)
-{
-	/* Verifying that the msgb context is empty */
-	talloc_report_full(msgb_ctx, stderr);
-	/* Expecting these to stick around in msc_vlr_tests_ctx:
-	 * full talloc report on 'msgb' (total      0 bytes in   1 blocks)
-	 * talloc_total_blocks(tall_bsc_ctx) == 19
-	 * full talloc report on 'msc_vlr_tests_ctx' (total   6532 bytes in  19 blocks)
-	 * struct osmo_gsup_client        contains    256 bytes in   1 blocks (ref 0) 0x56143306aa10
-	 * struct gsm_network             contains   4775 bytes in  11 blocks (ref 0) 0x5614330697e0
-	 * 	struct mgcp_client             contains    688 bytes in   1 blocks (ref 0) 0x56143306ad80
-	 * 	struct sccp_ran_inst           contains    152 bytes in   1 blocks (ref 0) 0x56143306ac80
-	 * 	struct sccp_ran_inst           contains    152 bytes in   1 blocks (ref 0) 0x56143306ab80
-	 * 	struct gsup_client_mux         contains    152 bytes in   2 blocks (ref 0) 0x56143306a8a0
-	 * 	struct ipaccess_unit           contains     64 bytes in   1 blocks (ref 0) 0x56143306a960
-	 * 	struct vlr_instance            contains    248 bytes in   1 blocks (ref 0) 0x56143306a740
-	 * 	no_gsup_server                 contains     15 bytes in   1 blocks (ref 0) 0x56143306a6c0
-	 * 	stat_item.c:96                 contains    144 bytes in   2 blocks (ref 0) 0x56143306a550
-	 * 	stat_item.c:118                contains     96 bytes in   1 blocks (ref 0) 0x56143306a5f0
-	 * 	rate_ctr.c:234                 contains   2352 bytes in   1 blocks (ref 0) 0x561433069bb0
-	 * logging                        contains   1501 bytes in   5 blocks (ref 0) 0x561433068fe0
-	 * 	struct log_target              contains    244 bytes in   2 blocks (ref 0) 0x561433069610
-	 * 	struct log_category            contains     76 bytes in   1 blocks (ref 0) 0x561433069720
-	 * 	struct log_info                contains   1256 bytes in   2 blocks (ref 0) 0x561433069050
-	 * 	struct log_info_cat            contains   1216 bytes in   1 blocks (ref 0) 0x5614330690e0
-	 * msgb                           contains      0 bytes in   1 blocks (ref 0) 0x561433068f70
-	 */
-	fprintf(stderr, "talloc_total_blocks(tall_bsc_ctx) == %zu\n",
-		talloc_total_blocks(msc_vlr_tests_ctx));
-	if (talloc_total_blocks(msc_vlr_tests_ctx) != 19)
-		talloc_report_full(msc_vlr_tests_ctx, stderr);
-	fprintf(stderr, "\n");
-}
-
 static struct {
 	bool verbose;
 	int run_test_nr;
@@ -1128,28 +1097,39 @@ static void handle_options(int argc, char **argv)
 	}
 }
 
-void *msgb_ctx = NULL;
-
 static void run_tests(int nr)
 {
 	int test_nr;
 
-	check_talloc(msgb_ctx, msc_vlr_tests_ctx);
-
 	nr--; /* arg's first test is 1, in here it's 0 */
 	for (test_nr = 0; msc_vlr_tests[test_nr]; test_nr++) {
+		size_t talloc_blocks_before_test;
+
 		if (nr >= 0 && test_nr != nr)
 			continue;
 
 		if (cmdline_opts.verbose)
 			fprintf(stderr, "(test nr %d)\n", test_nr + 1);
 
+		talloc_blocks_before_test = talloc_total_blocks(msc_vlr_tests_ctx);
+
 		msc_vlr_tests[test_nr]();
 
+		if (talloc_total_blocks(msc_vlr_tests_ctx) != talloc_blocks_before_test) {
+			fprintf(stderr, "ERROR: talloc leak: %zu blocks\n",
+				talloc_total_blocks(msc_vlr_tests_ctx) - talloc_blocks_before_test);
+			talloc_report_full(msc_vlr_tests_ctx, stderr);
+			fprintf(stderr, "\n");
+		}
+
+		if (talloc_total_blocks(msgb_ctx) > 1) {
+			fprintf(stderr, "ERROR: msgb leak:\n");
+			talloc_report_full(msgb_ctx, stderr);
+			fprintf(stderr, "\n");
+		}
+
 		if (cmdline_opts.verbose)
 			fprintf(stderr, "(test nr %d)\n", test_nr + 1);
-
-		check_talloc(msgb_ctx, msc_vlr_tests_ctx);
 	}
 }
 
@@ -1212,9 +1192,11 @@ int main(int argc, char **argv)
 	OSMO_ASSERT(osmo_stderr_target);
 	log_set_use_color(osmo_stderr_target, 0);
 	log_set_print_timestamp(osmo_stderr_target, 0);
-	log_set_print_filename(osmo_stderr_target, 0);
+	log_set_print_filename2(osmo_stderr_target, LOG_FILENAME_NONE);
 	log_set_print_category(osmo_stderr_target, 1);
+	log_set_print_category_hex(osmo_stderr_target, 0);
 	log_set_category_filter(osmo_stderr_target, DLSMS, 1, LOGL_DEBUG);
+	log_set_category_filter(osmo_stderr_target, DLMGCP, 0, LOGL_NOTICE);
 
 	if (cmdline_opts.verbose) {
 		log_set_print_filename2(osmo_stderr_target, LOG_FILENAME_BASENAME);
@@ -1254,6 +1236,5 @@ int main(int argc, char **argv)
 
 	printf("Done\n");
 
-	check_talloc(msgb_ctx, msc_vlr_tests_ctx);
 	return 0;
 }
